// LICENSE : MIT
"use strict";
/**
 * @return {string}
 */

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Expression = Expression;
exports.typeofName = typeofName;
function Expression(tagName, typeValue) {
    if (typeValue.type && typeValue.type === "NullableType") {
        // recursion
        var otherExpression = Expression(tagName, typeValue.expression);
        return "(" + tagName + " == null || " + otherExpression + ")";
    } else if (typeValue.type && typeValue.type === "NonNullableType") {
        // recursion
        var _otherExpression = Expression(tagName, typeValue.expression);
        return "(" + tagName + " != null && " + _otherExpression + ")";
    } else {
        var expectedType = typeofName(typeValue.name);
        if (expectedType == null) {
            var expectedName = typeValue.name;
            // if right-hand is undefined, return true
            // if right-hand is not function, return true
            // if right-hand is function && left instanceof right
            return "(\n                typeof " + expectedName + " === \"undefined\" || typeof " + expectedName + " !== \"function\" || " + tagName + " instanceof " + expectedName + "\n             )";
        } else {
            return "typeof " + tagName + " === \"" + expectedType + "\"";
        }
    }
}
// typeof
function typeofName(nodeTypeName) {
    switch (nodeTypeName) {
        case "Object":
        case "object":
            return "object";
        case "Function":
            return "function";
        case "string":
        case "String":
            return "string";
        case "number":
        case "Number":
            return "number";
        case "boolean":
        case "Boolean":
            return "boolean";
        case "undefined":
            return "undefined";
        case "Symbol":
        case "symbol":
            return "symbol";
        default:
            return;
    }
}
//# sourceMappingURL=Expression.js.map